@@ -203,6 +203,7 @@ typedef uint64_t u64;
 #define IPR_XLATE_DEV_FMT_RC(rc)	((((rc) & 127) == 51) ? -EIO : 0)
 #define IPR_TYPE_AF_DISK                     0xC
 #define IPR_TYPE_ADAPTER                     0x1f
+#define IPR_TYPE_ARRAY				   0x1f
 #define IPR_TYPE_EMPTY_SLOT                  0xff
 
 #define IPR_ACTIVE_OPTIMIZED                 0x0
@@ -295,9 +296,11 @@ extern char *tool_name;
 extern struct sysfs_dev *head_zdev;
 extern struct sysfs_dev *tail_zdev;
 extern enum system_p_mode power_cur_mode;
+extern int tool_init_retry;
 
 struct sysfs_dev {
-	char sysfs_device_name[PATH_MAX];
+	u64 device_id;
+	char ioa_pci_addr[16];
 	struct sysfs_dev *next, *prev;
 };
 
@@ -1481,6 +1484,12 @@ struct ipr_dev {
 	u32 is_suspend_cand:1;
 	u32 is_resume_cand:1;
 	u8 write_cache_policy:1;
+	u8 supports_4k:1;
+	u8 supports_5xx:1;
+	u8 read_c7:1;
+	u8 locked:1;
+	int lock_fd;
+	u32 format_timeout;
 	struct scsi_dev_data *scsi_dev_data;
 	struct ipr_dev *ses[IPR_DEV_MAX_PATHS];
 	struct ipr_res_addr res_addr[IPR_DEV_MAX_PATHS];
@@ -2600,7 +2609,9 @@ struct ipr_drive_elem_status {
 	u8 reserved5:1;
 	u8 fault_requested:1;
 	u8 fault_sensed:1;
-	u8 reserved6:4;
+	u8 reserved6:1;
+	u8 enable_byp:2;
+	u8 reserved7:1;
 	u8 disable_resets:1;
 #elif defined (__LITTLE_ENDIAN_BITFIELD)
 	u8 status:4;
@@ -2620,7 +2631,9 @@ struct ipr_drive_elem_status {
 	u8 reserved3:4;
 
 	u8 disable_resets:1;
-	u8 reserved6:4;
+	u8 reserved7:1;
+	u8 enable_byp:2;
+	u8 reserved6:1;
 	u8 fault_sensed:1;
 	u8 fault_requested:1;
 	u8 reserved5:1;
@@ -2692,9 +2705,17 @@ struct ipr_sas_inquiry_pageC7 {
 	u8 reserved1;
 	u8 page_length;
 	u8 ascii_len;
-	u8 reserved2[109];
+	u8 reserved2[38];
+	u8 format_timeout_hi; /* in minutes */
+	u8 format_timeout_lo;
+	u8 reserved3[63];
+	u8 support_4k_modes;
+#define ONLY_5XX_SUPPORTED		0
+#define BOTH_5XXe_OR_4K_SUPPORTED	1
+#define ONLY_4K_SUPPORTED		2
+	u8 reserved4[5];
 	u8 total_bytes_warranty[IPR_SAS_INQ_BYTES_WARRANTY_LEN];
-	u8 reserved3[43];
+	u8 reserved5[43];
 };
 
 static inline int ipr_elem_offset(struct ipr_ses_config_pg *ses_cfg, u8 type)
@@ -2934,6 +2955,14 @@ int ipr_get_live_dump(struct ipr_ioa *);
 int ipr_jbod_sysfs_bind(struct ipr_dev *, u8);
 int ipr_max_queue_depth(struct ipr_ioa *ioa, int num_devs, int num_ssd_devs);
 void ipr_count_devices_in_vset(struct ipr_dev *, int *num_devs, int *ssd_num_devs);
+int ipr_known_zeroed_is_saved(struct ipr_dev *);
+int get_sg_name(struct scsi_dev_data *);
+int ipr_sg_inquiry(struct scsi_dev_data *, u8, void *, u8);
+int get_scsi_dev_data(struct scsi_dev_data **scsi_dev_ref);
+
+int ipr_device_lock(struct ipr_dev *dev);
+void ipr_device_unlock(struct ipr_dev *dev);
+int check_sg_module();
 
 static inline u32 ipr_get_dev_res_handle(struct ipr_ioa *ioa, struct ipr_dev_record *dev_rcd)
 {
@@ -2978,6 +3007,15 @@ static inline int ipr_is_array_record(int record_id)
 		return 0;
 }
 
+static inline int ipr_is_ioa(struct ipr_dev *device)
+{
+	if (device->ioa && device->ioa->ioa.scsi_dev_data &&
+	    device->scsi_dev_data == device->ioa->ioa.scsi_dev_data)
+		return 1;
+	else
+		return 0;
+}
+
 static inline int ipr_is_af_dasd_device(struct ipr_dev *device)
 {
 	if ((device->qac_entry != NULL) &&
